unit BenchProgress;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Gauges, Db, DBTables;
const crlf = #13#10;

type
  TFormProgress = class(TForm)
    lbOverall: TLabel;
    gaOverall: TGauge;
    btStop: TButton;
    lbRecNo: TLabel;
    gaCurrent: TGauge;
    DataSource1: TDataSource;
    ConvertTable: TTable;
    Label1: TLabel;
    Label2: TLabel;
    lbRecCount: TLabel;
    Label3: TLabel;
    lbPassNo: TLabel;
    Label4: TLabel;
    lbPassCount: TLabel;
    TestTable: TTable;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btStopClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    bCancel: boolean;
    CurBenchCaption: string;
    log : string;

    // run all tests
    procedure RunTests;
    // initialization
    procedure InitTest;
    // creates and opens table
    procedure CreateTable;

    // basic input / output
    procedure RunBasicIO;
    // insert records
    procedure InsertRecords(showFlag : Boolean = true);
    // edit all records
    procedure EditRecords;
    // delete all records
    procedure DeleteRecords;
    // read mrec records (by 10 top-level directories)
    procedure ReadRecords;
    // moves cursor to the middle of the table
    procedure NavigateRecords;

    // input / output with BLOB fields
    procedure RunBlobsIO;
    // read blob data
    procedure ReadBLOBData;
    // delete records
    procedure DeleteBLOBData;
    // write blob data
    procedure WriteBLOBData(insertMode : Boolean = false);

    // FindFirst, FindNext
    procedure RunFind;
    // finds records by Integer
    procedure FindFirstTest1;
    // finds records by String
    procedure FindFirstTest2;
    // finds records by Date
    procedure FindFirstTest3;
    // finds records by combined condition
    procedure FindFirstTest4;
    // finds records by string with partial compare
    procedure FindFirstTest5;

    // finds records by Integer
    procedure FindNextTest1;
    // finds records by String
    procedure FindNextTest2;
    // finds records by Date
    procedure FindNextTest3;
    // finds records by combined condition
    procedure FindNextTest4;
    // finds records by string with partial compare
    procedure FindNextTest5;

    // Locate
    procedure RunLocate;
    // locates record by Locate method (integer)
    procedure LocateTest1;
    // locates record by Locate method (String)
    procedure LocateTest2;
    // locates record by Locate method (Date)
    procedure LocateTest3;
    // locates record by Locate method (all fields)
    procedure LocateTest4;
    // locates record by Locate method (substring)
    procedure LocateTest5;

    // Filtering
    procedure RunFilter;
    // integer
    procedure FilterTest1;
    // string
    procedure FilterTest2;
    // date
    procedure FilterTest3;
    // complex
    procedure FilterTest4;
    // substring
    procedure FilterTest5;

    // write log file
    procedure CloseTest;
  public
    { Public declarations }
    EWindowList:pointer;
    EActiveWindow:HWND;
    function MessageHook(var Msg: TMessage): Boolean;
    procedure Start;
    procedure SetProgress(RecNo,PassNo,RecCount,PassCount: integer);
  end;

function GenerateString(
                       len : Integer // serial length
                       ) : String; // returns serial
function MyTimeToString(
          time : LongInt) : string;

var
  FormProgress: TFormProgress;
  mrec : Integer; //records in table
  brec : Integer = 28; //blob records
  frec : Integer;
  passes : Cardinal = 1;
  mrec2 : Integer;
  mrec4 : Integer;
  trec : Integer; //test records
  drec : Integer; //offset for test records
  eqRec : Integer; // equal records for findNext and Filter test
  f,f1 : Extended;
  overallTime,avgCurrentTime, currentTime, avgTestTime, testTime : Cardinal;
  log : string;
  noteStreams : array of TMemoryStream;
  grStreams : array of TMemoryStream;
  dates : array of TDateTime;
  ints : array of Integer;
  strList : TStringList;
  ids : array of Integer;
  GlobalPassNo, GlobalPassCount: integer;

implementation

uses Main;

{$R *.DFM}

procedure TFormProgress.RunTests;
begin
 // benchmark groups count
 with MainForm do
  begin
   // Initialization
   InitTest;
   // create table
   CreateTable;
   // Basic I/O
   if (clbGroups.State[0] = cbChecked) then
    RunBasicIO;
   if bCancel then exit;
   // Blobs I/O
   if (clbGroups.State[1] = cbChecked) then
    RunBlobsIO;
   if bCancel then exit;
   // Find
   if (clbGroups.State[2] = cbChecked) then
    RunFind;
   if bCancel then exit;
   // Locate
   if (clbGroups.State[3] = cbChecked) then
    RunLocate;
   if bCancel then exit;
   // Filter
   if (clbGroups.State[4] = cbChecked) then
    RunFilter;
   if bCancel then exit;
  end;
end;

procedure TFormProgress.InitTest;
var i,d,m,y,h,n,z,j : integer;
    s : string;
begin
 Randomize;
 DateSeparator := '/';
 ShortTimeFormat := 'hh:nn:ss';
 ShortDateFormat := 'mm/dd/yyyy';
 mrec := MainForm.seQuantity.Value;
 if (mrec < 100) then
  mrec := 100;
 if (mrec >= 1000000) then
  mrec := 999999;

 mrec2 := mrec div 2;
 mrec4 := mrec div 4;
 if (mrec >= 1000) then
  begin
   frec := mrec div 10;
   drec := mrec2 - (frec shr 1);
  end
 else
  begin
   frec := mrec2;
   drec := mrec4;
  end;

 passes :=  MainForm.sePasses.Value;
 if (passes <= 0) then
  passes := 1;

 GlobalPassNo:=0;
 GlobalPassCount:=0;
 // Basic I/O
 if (MainForm.clbGroups.State[0] = cbChecked) then
  inc(GlobalPassCount,passes*6)
 else
  inc(GlobalPassCount);
 // Blobs I/O
 if (MainForm.clbGroups.State[1] = cbChecked) then
  inc(GlobalPassCount,passes*4);
 // Find
 if (MainForm.clbGroups.State[2] = cbChecked) then
  inc(GlobalPassCount,passes*10);
 // Locate
 if (MainForm.clbGroups.State[3] = cbChecked) then
  inc(GlobalPassCount,passes*5);
 // Filter
 if (MainForm.clbGroups.State[4] = cbChecked) then
  inc(GlobalPassCount,passes*5);

 lbRecCount.Caption := IntToStr(mrec);
 lbPassCount.Caption := IntToStr(passes);
 lbRecNo.Caption := '1';
 lbPassNo.Caption := '1';
 lbOverall.Caption := 'Initializing... Please, wait.';
 MainForm.reResults.Lines.Clear;
 log := '';

 strList := TStringList.Create;
 SetLength(ints,mrec);
 SetLength(ids,mrec);
 SetLength(dates,mrec);
 for i := 0 to mrec-1 do
  begin
     ids[i] := i+1;
     m:= 1 + Random(MAXINT) mod 12;
     d:= 1 + Random(MAXINT) mod 28;
     y := 1900 + Random(MAXINT) mod 200;
     h := Random(MAXINT) mod 24;
     n := Random(MAXINT) mod 60;
     z := Random(MAXINT) mod 60;

     s := inttostr(m) + '/' + inttostr(d) + '/' + inttostr(y) +
          ' ' + inttostr(h)+':' + inttostr(n)+':' + inttostr(z);
     dates[i] := StrToDateTime(s);
     s :=  'TeSt'+Format('%6d',[i+1])+GenerateString(90);
     strList.Add(s);
     ints[i] := Random(MAXINT) mod mrec;
    Application.ProcessMessages;
  end;
 z := Random(MAXINT) mod mrec;
 eqRec := z;
 j := 0;
 while j < frec do
  begin
   m := Random(MAXINT) mod mrec;
   if ((m = z) or
       (ints[m] = ints[z]) or
       (DateTimeToStr(dates[m]) = DateTimeToStr(dates[z])) or
       (strList.Strings[m] = strList.Strings[z])) then
       continue;
   ints[m] := ints[z];
   dates[m] := dates[z];
   strList.strings[m] := strList.strings[z];
   inc(j);
   Application.ProcessMessages;
  end;
end;


function TFormProgress.MessageHook(var Msg: TMessage): Boolean;
begin
  Result := False;
end;

procedure TFormProgress.Start;
begin
 gaOverall.Progress := 0;
 gaCurrent.Progress := 0;
 EActiveWindow := GetActiveWindow;
 Application.HookMainWindow(MessageHook);
 EWindowList := DisableTaskWindows(0);
 bCancel := false;
 Show;
 RunTests;
 Close;
end;

procedure TFormProgress.SetProgress(RecNo,PassNo,RecCount,PassCount: integer);
var f: double;
begin
 if (StrToInt(lbPassCount.Caption) <> PassCount) then
  lbPassCount.Caption := IntToStr(PassCount);
 if (StrToInt(lbRecCount.Caption) <> RecCount) then
  lbRecCount.Caption := IntToStr(RecCount);
 f := (RecNo-1) / (RecCount-1);
 gaCurrent.Progress := Round(100*f);
 gaOverall.Progress := Round(100*(GlobalPassNo-1+f)/GlobalPassCount);
end;

procedure TFormProgress.FormCreate(Sender: TObject);
begin
 TestTable.DatabaseName := ExtractFilePath(Application.ExeName) + 'Data';
 TestTable.TableName := 'test';
 ConvertTable.DatabaseName := ExtractFilePath(Application.ExeName) + '..\Data_pic';
 ConvertTable.TableName := 'test';

 MainForm.Caption := 'DB Engine Benchmark v.1.00 for Paradox';
end;

procedure TFormProgress.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
    Application.UnhookMainWindow(MessageHook);
    EnableTaskWindows(EWindowList);
    SetActiveWindow(EActiveWindow);
end;

procedure TFormProgress.btStopClick(Sender: TObject);
begin
 bCancel := true;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TFormProgress.CreateTable;
var i : integer;
    Stream1 : TStream;
begin
 with TestTable do
  begin
   Active := false;
   FieldDefs.Clear;
   FieldDefs.Add('ID', ftAutoInc, 0, false);
   FieldDefs.Add('Integer', ftInteger, 0, false);
   FieldDefs.Add('String', ftString, 100, False);
   FieldDefs.Add('Date', ftDateTime, 0, False);
   FieldDefs.Add('Notes',ftMemo,0,False);
   FieldDefs.Add('Graphic',ftGraphic,0,False);

   IndexDefs.Clear;
   IndexDefs.Add('ID_key','ID',[ixPrimary,ixUnique]);
   IndexDefs.Add('String_key','String;Date;Integer ',[ixCaseInsensitive]);
   IndexDefs.Add('Integer_key','Integer',[ixDescending]);
   IndexDefs.Add('Date_key','Date',[ixCaseInsensitive]);
   CreateTable;

  end; //with test table
  TestTable.Active := true;

  ConvertTable.Active := false;
  ConvertTable.Active := true;
  TestTable.First;
  ConvertTable.First;
  SetLength(grStreams,brec);
  SetLength(noteStreams,brec);
  for i := 0 to brec-1 do
   begin
    if (ConvertTable.Eof) then
      begin
         brec := i+1;
         break;
      end;
    if (grStreams[i] <> nil) then
      begin
         grStreams[i].Free;
         noteStreams[i].Free;
      end;
    grStreams[i] := TMemoryStream.Create;
    noteStreams[i] := TMemoryStream.Create;
    Stream1 := ConvertTable.CreateBlobStream(ConvertTable.FieldByName('Notes'), bmRead);
    noteStreams[i].CopyFrom(Stream1, Stream1.Size);
    Stream1.Free;

    Stream1 := ConvertTable.CreateBLOBStream(ConvertTable.FieldByName('Graphic'), bmRead);
    grStreams[i].CopyFrom(Stream1, Stream1.Size);
    Stream1.Free;

    ConvertTable.Next;
    Application.ProcessMessages;
   end;
end; //CreateTable

//------------------------------------------------------------------------------
// basic input/output benchmark
//------------------------------------------------------------------------------
procedure TFormProgress.RunBasicIO;
begin
 CurBenchCaption := 'Basic Input/Output';
 log := log + 'Basic Input/Output benchmark...,'+crlf+
        'RecordCount = '+inttostr(mrec)+', selection RecordCount = '+
        inttostr(frec)+crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//------------------------------------- insert ---------------------------------
   try
     InsertRecords;
   except
    log := log + 'Error in InsertRecords!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 if bCancel then exit;
//------------------------------------- edit ---------------------------------
   try
     EditRecords;
   except
    log := log + 'Error in EditRecords!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 if bCancel then exit;
//------------------------------------- delete ---------------------------------
   try
     DeleteRecords;
   except
    log := log + 'Error in DeleteRecords!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 if bCancel then exit;
//------------------------------------- read -----------------------------------
   try
     ReadRecords;
   except
    log := log + 'Error in ReadRecords!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 if bCancel then exit;
//------------------------------------- navigate -------------------------------
   try
     NavigateRecords;
   except
    log := log + 'Error in NavigateRecords!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 CloseTest;
end; // basic input/output benchmark


//------------------------------------------------------------------------------
// insert records
//------------------------------------------------------------------------------
procedure TFormProgress.InsertRecords(showFlag : Boolean = true);
var i,j,k : integer;
begin
 if (showFlag) then
  begin
   avgCurrentTime := 0;
   avgTestTime := 0;
   SetProgress(0,1,mrec,passes);
   lbOverall.Caption := CurBenchCaption+': Inserting records'
  end
 else
  begin
   SetProgress(0,1,mrec,1);
   lbOverall.Caption := CurBenchCaption+': Inserting records';
  end;
 Application.ProcessMessages();
 trec := mrec;
 f := trec;
 with TestTable do
  begin
   if (showFlag) then
    k := passes
   else
    k := 1;
   for i := 1 to k do
    begin
     inc(GlobalPassNo);
     EmptyTable;
     currentTime := 0;
     for j := 0 to mrec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       Insert;
//       FieldByName('ID').AsInteger := ids[j];
       FieldByName('Integer').AsInteger := ints[j];
       FieldByName('String').AsString := strList.strings[j];
       FieldByName('Date').AsDateTime := dates[j];
       Post;
       // stop timer
       testTime := GetTickCount-testTime;
       if (showFlag) then
        begin
         inc(currentTime,testTime);
        end;
       // progress
       SetProgress(j+1,i,mrec,k);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if (not showFlag) then Exit;
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Insert records test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //InsertRecords;


//------------------------------------------------------------------------------
// edit records
//------------------------------------------------------------------------------
procedure TFormProgress.EditRecords;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 lbOverall.Caption := CurBenchCaption+': Editing records';
 Application.ProcessMessages();
 trec := mrec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     inc(GlobalPassNo);
     First;
     currentTime := 0;
     Application.ProcessMessages();
     for j := 0 to mrec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       Edit;
//       FieldByName('ID').AsInteger := ids[j];
       FieldByName('String').AsString := strList.strings[j];
       FieldByName('Integer').AsInteger := ints[j];
       FieldByName('Date').AsDateTime := dates[j];
       Post;
       // stop timer
       testTime := GetTickCount-testTime;
       Next;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,mrec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Edit records test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //EditRecords;


//------------------------------------------------------------------------------
// delete records
//------------------------------------------------------------------------------
procedure TFormProgress.DeleteRecords;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := mrec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     inc(GlobalPassNo);
     lbOverall.Caption := CurBenchCaption+': Deleting records';
     Application.ProcessMessages();
     First;
     currentTime := 0;
     Application.ProcessMessages();
     for j := 0 to mrec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       Delete;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,mrec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Delete records test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //DeleteRecords;


//------------------------------------------------------------------------------
// read records
//------------------------------------------------------------------------------
procedure TFormProgress.ReadRecords;
var i,j,k : integer;
    s : string;
    d : TDateTime;
begin
 if (TestTable.RecordCount < mrec) then
   InsertRecords(false);
 avgCurrentTime := 0;
 avgTestTime := 0;
 lbOverall.Caption := CurBenchCaption+': Reading records';
 Application.ProcessMessages();
 trec := mrec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     inc(GlobalPassNo);
     First;
     Filtered := false;
     currentTime := 0;
     Application.ProcessMessages();
     for j := 0 to mrec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       k := FieldByName('ID').AsInteger;
       s := FieldByName('String').AsString;
       k := FieldByName('Integer').AsInteger;
       d := FieldByName('Date').AsDateTime;
       // stop timer
       testTime := GetTickCount-testTime;
       Next;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,mrec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Read records test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //ReadRecords;


//------------------------------------------------------------------------------
// navigate records
//------------------------------------------------------------------------------
procedure TFormProgress.NavigateRecords;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 lbOverall.Caption := CurBenchCaption+': Navigating records';
 Application.ProcessMessages();
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     inc(GlobalPassNo);
     currentTime := 0;
     Application.ProcessMessages();
     for j := 0 to frec-1 do
      begin
       testTime := GetTickCount;
       First;
       // start timer
       MoveBy(drec + j);
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Navigate records test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //MoveBy;


//------------------------------------------------------------------------------
// BLOB benchmark
//------------------------------------------------------------------------------
procedure TFormProgress.RunBlobsIO;
var j : integer;
label fin;
begin
 CurBenchCaption := 'Blobs Input/Output';
 log := log+crlf+'BLOB data benchmark...,'+crlf+
        'RecordCount = '+inttostr(mrec)+', selection RecordCount = '+
        inttostr(mrec)+crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 //---------------------------------- BLOB write ---------------------------------
   try
     WriteBLOBData;
   except
    log := log + 'Error in WriteBLOBData!'+crlf;
    CloseTest;
   end;
 if bCancel then goto fin;
//---------------------------------- BLOB delete -------------------------------
   try
     ReadBLOBData;
   except
    log := log + 'Error in ReadBLOBData!'+crlf;
    CloseTest;
   end;
 if bCancel then goto fin;

//---------------------------------- BLOB write --------------------------------
   try
     DeleteBLOBData;
   except
    log := log + 'Error in DeleteBLOBData!'+crlf;
    CloseTest;
   end;
 CloseTest;

fin:
  for j := 0 to brec-1 do
   if (grStreams[j] <> nil) then
    begin
     grStreams[j].Free;
     noteStreams[j].Free;
    end;
 grStreams := nil;
 noteStreams := nil;
end; // BLOB benchmark


//------------------------------------------------------------------------------
// read blob data
//------------------------------------------------------------------------------
procedure TFormProgress.ReadBLOBData;
var i,j,k : integer;
    Stream1 : TStream;
    Stream2 : TStream;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 lbOverall.Caption := CurBenchCaption+': Read BLOB Data';
 Application.ProcessMessages();
 trec := mrec;
 f := mrec;
   for i := 1 to passes do
    begin
     if (TestTable.RecordCount < mrec) then
      WriteBLOBData(true);
     inc(GlobalPassNo);
     currentTime := 0;
     TestTable.First;
     for j := 0 to mrec-1 do
      begin
       try
       k := j mod brec;
       if (grStreams[k] <> nil) then
       begin
        grStreams[k].Free;
        noteStreams[k].Free;
       end;
        grStreams[k] := TMemoryStream.Create;
        noteStreams[k] := TMemoryStream.Create;
        // start timer
        testTime := GetTickCount;
        stream1 := TestTable.CreateBLOBStream(TestTable.FieldByName('Notes'),bmRead);
        noteStreams[k].CopyFrom(Stream1, Stream1.Size);
        stream1.Free;
        stream2 := TestTable.CreateBLOBStream(TestTable.FieldByName('Graphic'),bmRead);
        grStreams[k].CopyFrom(Stream2, Stream2.Size);
        stream2.Free;
        // stop timer
        testTime := GetTickCount-testTime;
       except
        TestTable.Active := false;
        ShowMessage('Error while reading blob record #'+inttostr(i));
       end;
       TestTable.Next;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,mrec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Read BLOB Data'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; // read blob data


//------------------------------------------------------------------------------
// delete blob data
//------------------------------------------------------------------------------
procedure TFormProgress.DeleteBLOBData;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := mrec;
 f := mrec;
   for i := 1 to passes do
    begin
     if (TestTable.RecordCount < mrec) then
      //insert records
      WriteBLOBData(true);
     lbOverall.Caption := CurBenchCaption+': Delete BLOB Data';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     currentTime := 0;
     TestTable.First;
     for j := 0 to mrec-1 do
      begin
       // start timer
       testTime := GetTickCount;
       TestTable.Delete;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,mrec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Delete BLOB Data'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; // delete blob data


//------------------------------------------------------------------------------
// write blob data
//------------------------------------------------------------------------------
procedure TFormProgress.WriteBLOBData(insertMode : Boolean = false);
var i,j,k,kk : integer;
    Stream1 : TStream;
    Stream2 : TStream;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 lbOverall.Caption := CurBenchCaption+': Insert BLOB Data';
 Application.ProcessMessages();
 trec := mrec;
 f := mrec;
 if (insertMode) then
    kk := 1
 else
    kk := passes;
 for i := 1 to kk do
    begin
     CreateTable;
     inc(GlobalPassNo);
     currentTime := 0;
     TestTable.First;
     for j := 0 to mrec-1 do
      begin
       try
        k := j mod brec;
        noteStreams[k].Seek(0,soFromBeginning);
        grStreams[k].Seek(0,soFromBeginning);
        // start timer
        testTime := GetTickCount;
        TestTable.Insert;
//        TestTable.FieldByName('ID').AsInteger := ids[j];
        TestTable.FieldByName('Integer').AsInteger := ints[j];
        TestTable.Post;
        TestTable.Edit;

        stream1 := TestTable.CreateBLOBStream(TestTable.FieldByName('Notes'),bmReadWrite);
        stream1.CopyFrom(noteStreams[k], noteStreams[k].Size);
        stream1.Free;

        stream2 := TestTable.CreateBLOBStream(TestTable.FieldByName('Graphic'),bmReadWrite);
        stream2.CopyFrom(grStreams[k], grStreams[k].Size);
        stream2.Free;

        TestTable.Post;
        // stop timer
        testTime := GetTickCount-testTime;
       except
        TestTable.Active := false;
        ShowMessage('Error while inserting blob record #'+inttostr(j));
       end;
      if not insertMode then
       inc(currentTime,testTime);
      TestTable.Next;
       // progress
       SetProgress(j+1,i,mrec,kk);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
 if (insertMode) then
  exit;
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Insert BLOB Data'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //write BLOB data


//------------------------------------------------------------------------------
// FindFirst, FindNext
//------------------------------------------------------------------------------
procedure TFormProgress.RunFind;
begin
 CurBenchCaption := 'Finding';
 log := log+crlf+'Find benchmark...,'+crlf+
        'RecordCount = '+inttostr(mrec)+', selection RecordCount = '+
        inttostr(frec)+crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindFirst integer --------------------------------
   try
     FindFirstTest1;
   except
    log := log + 'Error in FindFirst1!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindFirst string ---------------------------------
   try
     FindFirstTest2;
   except
    log := log + 'Error in FindFirst2!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindFirst date -----------------------------------
   try
     FindFirstTest3;
   except
    log := log + 'Error in FindFirst3!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//------------------- FindFirst complicated condition --------------------------
   try
     FindFirstTest4;
   except
    log := log + 'Error in FindFirst4!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindFirst substring ------------------------------
   try
     FindFirstTest5;
   except
    log := log + 'Error in FindFirst5!'+crlf;
    CloseTest;
   end;
 log := log + crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;

//--------------------------- FindNext integer --------------------------------
   try
     FindNextTest1;
   except
    log := log + 'Error in FindNext1!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindNext string ---------------------------------
   try
     FindNextTest2;
   except
    log := log + 'Error in FindNext2!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindNext date -----------------------------------
   try
     FindNextTest3;
   except
    log := log + 'Error in FindNext3!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//------------------- FindNext complicated condition --------------------------
   try
     FindNextTest4;
   except
    log := log + 'Error in FindNext4!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- FindNext substring ------------------------------
   try
     FindNextTest5;
   except
    log := log + 'Error in FindNext5!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
CloseTest;
end;//TFormProgress.RunFind


//------------------------------------------------------------------------------
// find integer field
//------------------------------------------------------------------------------
procedure TFormProgress.FindFirstTest1;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindFirst for integer field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       Filter := 'Integer = '+inttostr(ints[k]);
       if (not FindFirst) then
            raise Exception.Create('FindFirst: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('Integer').AsInteger <> ints[k]) then
            raise Exception.Create('FindFirst: invalid integer = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindFirst integer test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end;


//------------------------------------------------------------------------------
// find string field
//------------------------------------------------------------------------------
procedure TFormProgress.FindFirstTest2;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindFirst for string field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       Filter := 'String = '+
          LowerCase(AnsiQuotedStr(strList.strings[k],''''));
       FilterOptions := [foCaseInsensitive,foNoPartialCompare];
       if (not FindFirst) then
            raise Exception.Create('FindFirst: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[k]) then
            raise Exception.Create('FindFirst: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindFirst string test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest2;


//------------------------------------------------------------------------------
// find DateTime field
//------------------------------------------------------------------------------
procedure TFormProgress.FindFirstTest3;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindFirst for datetime field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       First;
       // start timer
       testTime := GetTickCount;
       FilterOptions := [foCaseInsensitive,foNoPartialCompare];
       Filter := 'Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[k]),'''');
       if (not FindFirst) then
            raise Exception.Create('FindFirst: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[k])) then
            raise Exception.Create('FindFirst: invalid date = '+
            FieldByName('Date').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindFirst datetime test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest3;


//------------------------------------------------------------------------------
// find records by complicated condition
//------------------------------------------------------------------------------
procedure TFormProgress.FindFirstTest4;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindFirst for combined condition';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       First;
       // start timer
       testTime := GetTickCount;
       FilterOptions := [foCaseInsensitive,foNoPartialCompare];
       Filter :=
          '(String = '+
          LowerCase(AnsiQuotedStr(strList.strings[k],'''')) +') and'+
          '(Integer = '+inttostr(ints[k])+') and '+
          '(Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[k]),'''')+ ')';
       if (not FindFirst) then
            raise Exception.Create('FindFirst: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if ((FieldByName('Integer').AsInteger <> ints[k]) or
           (FieldByName('String').AsString <> strList.strings[k]) or
           (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[k])))
         then
            raise Exception.Create('FindFirst: invalid record found RecNo = '+
            inttostr(RecNo)+', k = '+inttostr(k)+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindFirst all fields test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest4;


//------------------------------------------------------------------------------
// find records by substring of string
//------------------------------------------------------------------------------
procedure TFormProgress.FindFirstTest5;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindFirst for substring';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       Filter := 'String = '+
          AnsiQuotedStr('test'+Format('%6d',[k+1])+'*','''');
       FilterOptions := [foCaseInsensitive];
       if (not FindFirst) then
            raise Exception.Create('FindFirst: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[k]) then
            raise Exception.Create('FindFirst: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindFirst substring test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest5;


//------------------------------------------------------------------------------
// find integer field
//------------------------------------------------------------------------------
procedure TFormProgress.FindNextTest1;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindNext for integer field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filter := 'Integer = '+inttostr(ints[eqRec]);
     if (not FindFirst) then
            raise Exception.Create('FindNext: not found, filter = '+Filter);
     for j := 0 to frec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       if (not FindNext) then
            raise Exception.Create('FindNext: not found2, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('Integer').AsInteger <> ints[eqRec]) then
            raise Exception.Create('FindNext: invalid integer = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindNext integer test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end;


//------------------------------------------------------------------------------
// find string field
//------------------------------------------------------------------------------
procedure TFormProgress.FindNextTest2;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindNext for string field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filter := 'String = '+
          LowerCase(AnsiQuotedStr(strList.strings[eqRec],''''));
     FilterOptions := [foCaseInsensitive,foNoPartialCompare];
     if (not FindFirst) then
            raise Exception.Create('FindNext: not found, filter = '+Filter);
     for j := 0 to frec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       if (not FindNext) then
            raise Exception.Create('FindNext: not found2, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[eqRec]) then
            raise Exception.Create('FindNext: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindNext string test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest2;


//------------------------------------------------------------------------------
// find DateTime field
//------------------------------------------------------------------------------
procedure TFormProgress.FindNextTest3;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindNext for DateTime field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     FilterOptions := [foCaseInsensitive,foNoPartialCompare];
     Filter := 'Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[eqRec]),'''');
     if (not FindFirst) then
            raise Exception.Create('FindNext: not found, filter = '+Filter);
     for j := 0 to frec-1 do
      begin
       // start timer
       testTime := GetTickCount;
       if (not FindNext) then
            raise Exception.Create('FindNext: not found2, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[eqRec])) then
            raise Exception.Create('FindNext: invalid date = '+
            FieldByName('Date').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindNext datetime test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest3;


//------------------------------------------------------------------------------
// find records by complicated condition
//------------------------------------------------------------------------------
procedure TFormProgress.FindNextTest4;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindNext for combined condition';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filter :=
          '(Integer = '+inttostr(ints[eqRec])+') and '+
          '(String = '+
          LowerCase(AnsiQuotedStr(strList.strings[eqRec],'''')) +') and'+
          '(Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[eqRec]),'''')+ ')';
     FilterOptions := [foCaseInsensitive,foNoPartialCompare];
     if (not FindFirst) then
            raise Exception.Create('FindNext: not found, filter = '+Filter);
     for j := 0 to frec-1 do
      begin
       testTime := GetTickCount;
       // start timer
       if (not FindNext) then
            raise Exception.Create('FindNext: not found2, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if ((FieldByName('Integer').AsInteger <> ints[eqRec]) or
           (FieldByName('String').AsString <> strList.strings[eqRec]) or
           (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[eqRec])))
         then
            raise Exception.Create('FindNext: invalid record found RecNo = '+
            inttostr(RecNo)+', eqRec = '+inttostr(eqRec)+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindNext all fields test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest4;


//------------------------------------------------------------------------------
// find records by substring of string
//------------------------------------------------------------------------------
procedure TFormProgress.FindNextTest5;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 Application.ProcessMessages();
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': FindNext for substring';
     Application.ProcessMessages();
     inc(GlobalPassNo);

     Filter := 'String = '+
          AnsiQuotedStr('test'+Format('%6d',[eqRec+1])+'*','''');
     FilterOptions := [foCaseInsensitive];
     if (not FindFirst) then
            raise Exception.Create('FindNext: not found, filter = '+Filter);
     for j := 0 to frec-1 do
      begin
       // start timer
       testTime := GetTickCount;
       if (not FindNext) then
            raise Exception.Create('FindNext: not found2, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[eqRec]) then
            raise Exception.Create('FindNext: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'FindNext substring test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FindTest5;


//------------------------------------------------------------------------------
// Locate
//------------------------------------------------------------------------------
procedure TFormProgress.RunLocate;
begin
 CurBenchCaption := 'Locating';
 log := log+crlf+'Locate benchmark...,'+crlf+
        'RecordCount = '+inttostr(mrec)+', selection RecordCount = '+
        inttostr(frec)+crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;

//--------------------------- Locate integer --------------------------------
   try
     LocateTest1;
   except
    log := log + 'Error in Locate1!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Locate string ---------------------------------
   try
     LocateTest2;
   except
    log := log + 'Error in Locate2!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Locate date -----------------------------------
   try
     LocateTest3;
   except
    log := log + 'Error in Locate3!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//------------------- Locate complicated condition --------------------------
   try
     LocateTest4;
   except
    log := log + 'Error in Locate4!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;

//--------------------------- Locate substring ------------------------------
   try
     LocateTest5;
   except
    log := log + 'Error in Locate5!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 CloseTest;
end;//TFormProgress.RunLocate;


//------------------------------------------------------------------------------
// locates record by Locate method (Integer)
//------------------------------------------------------------------------------
procedure TFormProgress.LocateTest1;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Locate for integer field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       if (not Locate('Integer',ints[k],[])) then
            raise Exception.Create('Locate: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('Integer').AsInteger <> ints[k]) then
            raise Exception.Create('Locate: invalid integer = '+
            FieldByName('Integer').AsString);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Locate integer test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //LocateTest1;


//------------------------------------------------------------------------------
// locates record by Locate method (String)
//------------------------------------------------------------------------------
procedure TFormProgress.LocateTest2;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Locate for string field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       if (not Locate('String',strList.strings[k],[loCaseInsensitive])) then
            raise Exception.Create('Locate: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[k]) then
            raise Exception.Create('Locate: invalid string = '+
            FieldByName('String').AsString);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Locate string test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //LocateTest2;


//------------------------------------------------------------------------------
// locates record by Locate method (creation_datre)
//------------------------------------------------------------------------------
procedure TFormProgress.LocateTest3;
var i,k,j : integer;
    v : Variant;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 Application.ProcessMessages();
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Locate for DateTime field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       if (not Locate('Date',dates[k],[])) then
            raise Exception.Create('Locate: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[k])) then
            raise Exception.Create('Locate: invalid date = '+
            FieldByName('Date').AsString+' should be = '+DateTimeToStr(dates[k]));
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Locate datetime test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //LocateTest3;


//------------------------------------------------------------------------------
// locates record by Locate method (all fields)
//------------------------------------------------------------------------------
procedure TFormProgress.LocateTest4;
var i,j,k : integer;
    v : Variant;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 V := VarArrayCreate([0,2], varVariant);
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Locate for all fields';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       v[0] := strList.strings[k];
       v[1] := ints[k];
       v[2] := dates[k];
       // start timer
       testTime := GetTickCount;
       if (not Locate('String;Integer;Date',  v,[loCaseInsensitive])) then
            raise Exception.Create('Locate: not found, filter = '+Filter);

       // stop timer
       testTime := GetTickCount-testTime;
       if ((FieldByName('Integer').AsInteger <> ints[k]) or
           (FieldByName('String').AsString <> strList.strings[k]) or
           (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[k])))
         then
            raise Exception.Create('Locate: invalid record found RecNo = '+
            inttostr(RecNo)+', k = '+inttostr(k));
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Locate all fields test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //LocateTest4;


//------------------------------------------------------------------------------
// find records by substring of string
//------------------------------------------------------------------------------
procedure TFormProgress.LocateTest5;
var i,j,k : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Locate for substring';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     for j := 0 to frec-1 do
      begin
       First;
       repeat
        k := Random(MAXINT) mod mrec;
       until (strList.strings[k] <> strList.strings[eqRec]);
       // start timer
       testTime := GetTickCount;
       if (not Locate('String',strList.strings[k],[loCaseInsensitive,loPartialKey])) then
            raise Exception.Create('Locate: not found, filter = '+Filter);
       // stop timer
       testTime := GetTickCount-testTime;
       if (FieldByName('String').AsString <> strList.strings[k]) then
            raise Exception.Create('Locate: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Locate substring test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //Locate5;


//------------------------------------------------------------------------------
// Filtering
//------------------------------------------------------------------------------
procedure TFormProgress.RunFilter;
begin
 CurBenchCaption := 'Filtering';
 log := log+crlf+'Filter benchmark...,'+crlf+
        'RecordCount = '+inttostr(mrec)+', selection RecordCount = '+
        inttostr(frec)+crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Filter integer -----------------------------------
   try
     FilterTest1;
   except
    log := log + 'Error in Filter1!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Filter string ------------------------------------
   try
     FilterTest2;
   except
    log := log + 'Error in Filter2!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Filter date---------------------------------------
   try
     FilterTest3;
   except
    log := log + 'Error in Filter3!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Filter all ---------------------------------------
   try
     FilterTest4;
   except
    log := log + 'Error in Filter4!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
//--------------------------- Filter substring ---------------------------------
   try
     FilterTest5;
   except
    log := log + 'Error in Filter5!'+crlf;
    CloseTest;
   end;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
 CloseTest;
end;//TFormProgress.RunFilter


//------------------------------------------------------------------------------
// filter integer field
//------------------------------------------------------------------------------
procedure TFormProgress.FilterTest1;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Filter for integer field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filtered := false;
     // start timer
     testTime := GetTickCount;
     Filter := 'Integer = '+inttostr(ints[eqRec]);

     if (not FindFirst) then
            raise Exception.Create('Filter: not found, filter = '+Filter);

     Filtered := true;
     First;
     // stop timer
     testTime := GetTickCount-testTime;
     if (Bof  and Eof) then
            raise Exception.Create('Filter: invalid integer0 = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
     inc(currentTime,testTime);
     for j := 0 to frec-1 do
      begin
       if (Eof) then break;
       if (FieldByName('Integer').AsInteger <> ints[eqRec]) then
            raise Exception.Create('Filter: invalid integer = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);

       // start timer
       testTime := GetTickCount;
       Next;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
     Filtered := false;
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Filter integer test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FilterTest1


//------------------------------------------------------------------------------
// filter string field
//------------------------------------------------------------------------------
procedure TFormProgress.FilterTest2;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Filter for string field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filtered := false;
     // start timer
     testTime := GetTickCount;
     Filter := 'String = '+AnsiQuotedStr(strList.strings[eqRec],'''');

     if (not FindFirst) then
            raise Exception.Create('Filter: not found, filter = '+Filter);

     Filtered := true;
     First;
     // stop timer
     testTime := GetTickCount-testTime;
     if (Bof  and Eof) then
            raise Exception.Create('Filter: invalid string0 = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
     inc(currentTime,testTime);
     for j := 0 to frec-1 do
      begin
       if (Eof) then break;
       if (FieldByName('String').AsString <> strList.strings[eqRec]) then
            raise Exception.Create('Filter: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);

       // start timer
       testTime := GetTickCount;
       Next;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
     Filtered := false;
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Filter string test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FilterTest1


//------------------------------------------------------------------------------
// filter date field
//------------------------------------------------------------------------------
procedure TFormProgress.FilterTest3;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Filter for DateTime field';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filtered := false;
     // start timer
     testTime := GetTickCount;

     Filter := 'Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[eqRec]),'''');

     if (not FindFirst) then
            raise Exception.Create('Filter: not found, filter = '+Filter);

     Filtered := true;
     First;
     // stop timer
     testTime := GetTickCount-testTime;
     if (Bof  and Eof) then
            raise Exception.Create('Filter: invalid date0 = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
     inc(currentTime,testTime);
     for j := 0 to frec-1 do
      begin
       if (Eof) then break;
       if (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[eqRec])) then
            raise Exception.Create('Filter: invalid date = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);

       // start timer
       testTime := GetTickCount;
       Next;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
     Filtered := false;
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Filter date test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FilterTest3


//------------------------------------------------------------------------------
// filter complex
//------------------------------------------------------------------------------
procedure TFormProgress.FilterTest4;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Filter for combined condition';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filtered := false;
     // start timer
     testTime := GetTickCount;
     FilterOptions := [foCaseInsensitive];
       Filter :=
          '(String = '+
          LowerCase(AnsiQuotedStr(strList.strings[eqRec],'''')) +') and'+
          '(Integer = '+inttostr(ints[eqRec])
          +') and '+
          '(Date = '+
          AnsiQuotedStr(DateTimeToStr(dates[eqRec]),'''')+ ')';

     if (not FindFirst) then
            raise Exception.Create('Filter: not found, filter = '+Filter);

     Filtered := true;
     First;
     // stop timer
     testTime := GetTickCount-testTime;
     if (Bof  and Eof) then
            raise Exception.Create('Filter: invalid all fields0 = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
     inc(currentTime,testTime);
     for j := 0 to frec-1 do
      begin
       if (Eof) then break;
       if (FieldByName('Integer').AsInteger <> ints[eqRec]) then
            raise Exception.Create('Filter: invalid integer = '+
            FieldByName('Integer').AsString+
            ', filter = '+Filter);
       if (FieldByName('String').AsString <> strList.Strings[eqRec]) then
            raise Exception.Create('Filter: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
       if (DateTimeToStr(FieldByName('Date').AsDateTime) <> DateTimeToStr(dates[eqRec])) then
            raise Exception.Create('Filter: invalid date = '+
            FieldByName('Date').AsString+
            ', filter = '+Filter);

       // start timer
       testTime := GetTickCount;
       Next;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
     Filtered := false;
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Filter all fields test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FilterTest4


//------------------------------------------------------------------------------
// filter substring
//------------------------------------------------------------------------------
procedure TFormProgress.FilterTest5;
var i,j : integer;
begin
 avgCurrentTime := 0;
 avgTestTime := 0;
 trec := frec;
 f := trec;
 with TestTable do
  begin
   for i := 1 to passes do
    begin
     if (RecordCount < mrec) then
      InsertRecords(false);
     currentTime := 0;
     lbOverall.Caption := CurBenchCaption+': Filter for substring';
     Application.ProcessMessages();
     inc(GlobalPassNo);
     Filtered := false;
     // start timer
     testTime := GetTickCount;
     Filter := 'String = '+
          AnsiQuotedStr('test'+Format('%6d',[eqRec+1])+'*','''');
     FilterOptions := [foCaseInsensitive];

     if (not FindFirst) then
            raise Exception.Create('Filter: not found, filter = '+Filter);

     Filtered := true;
     First;
     // stop timer
     testTime := GetTickCount-testTime;
     if (Bof  and Eof) then
            raise Exception.Create('Filter: invalid substring0 = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);
     inc(currentTime,testTime);
     for j := 0 to frec-1 do
      begin
       if (Eof) then break;
       if (FieldByName('String').AsString <> strList.strings[eqRec]) then
            raise Exception.Create('Filter: invalid string = '+
            FieldByName('String').AsString+
            ', filter = '+Filter);

       // start timer
       testTime := GetTickCount;
       Next;
       // stop timer
       testTime := GetTickCount-testTime;
       inc(currentTime,testTime);
       // progress
       SetProgress(j+1,i,frec,passes);
       lbRecNo.Caption := IntToStr(j+1);
       lbPassNo.Caption := IntToStr(i);
       Application.ProcessMessages();
       if bCancel then exit;
      end; // records
     inc (avgCurrentTime,currentTime);
     inc (avgTestTime,testTime);
     Filtered := false;
    end; // passes
  end; //with
 if ((avgCurrentTime mod passes) > (passes shr 1)) then
  avgCurrentTime := avgCurrentTime div passes + 1
 else
  avgCurrentTime := avgCurrentTime div passes;

 if ((avgTestTime mod passes) > (passes shr 1)) then
  avgTestTime := avgTestTime div passes + 1
 else
  avgTestTime := avgTestTime div passes;
 log :=  log + 'Filter substring test'+
        ','+MyTimeToString(avgCurrentTime)+
        crlf;
 MainForm.reResults.Lines.Clear;
 MainForm.reResults.Lines.Add(log);
 Application.ProcessMessages;
end; //FilterTest5


//------------------------------------------------------------------------------
// Save log file with results
//------------------------------------------------------------------------------
procedure TFormProgress.CloseTest;
var fileStream : TFileStream;
begin
   fileStream := TFileStream.Create('log.csv',fmCreate);
   MainForm.reResults.Lines.Clear;
   MainForm.reResults.Lines.Add(log);
   fileStream.WriteBuffer(PChar(log)^,Length(log));
   fileStream.Free;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

function GenerateString(
                       len : Integer // serial length
                       ) : String; // returns serial
var i,x : integer;
    s : string;
    c : char;
begin
 s := '';

 for i := 1 to len do
  begin
   x := Random(101);
   if ((x mod 2) =  0) then
    c := chr(65+(Random(260000000) mod 26))
   else
    c := chr(48+(Random(100000000) mod 10));

   s := s + c;
  end; //len
 result := s;
end; // GenerateString

function MyTimeToString(
          time : LongInt) : string;
var
    sec,min,s1000 : LongInt;
begin
 s1000 := time mod 1000;
 sec := (time - s1000) div 1000;
 min := sec div 60;
 sec := sec mod 60;
 Result := Format(' %3u min %2u sec %3u ms',[min,sec,s1000]);
end;




end.
